%define anolis_release 2
Name:           shadow-utils
Version:        4.13
Release:        %{anolis_release}%{?dist}
Summary:        Utilities for managing accounts and shadow password files

License:        BSD and GPLv2+
URL:            https://github.com/shadow-maint/shadow
Source0:        https://github.com/shadow-maint/shadow/releases/download/%{version}/shadow-%{version}.tar.xz
Source1:        shadow-utils.useradd
Source2:        shadow-utils.login.defs
Source3:        shadow-bsd.txt
Source4:        https://www.gnu.org/licenses/old-licenses/gpl-2.0.txt

### Globals ###
%global includesubiddir %{_includedir}/shadow

### Patches ###
# Misc small changes - most probably non-upstreamable
Patch0: shadow-4.12.3-nologin.patch
# SElinux related - upstreamability unknown
Patch1: shadow-4.13-default-range.patch
# Misc manual page changes - non-upstreamable
Patch2: shadow-4.9-manfix.patch
# Date parsing improvement - could be upstreamed
Patch3: shadow-4.2.1-date-parsing.patch
# Audit message changes - upstreamability unknown
Patch4: shadow-4.11.1-audit-update.patch
# Changes related to password unlocking - could be upstreamed
Patch5: shadow-4.5-usermod-unlock.patch
# Additional SElinux related changes - upstreamability unknown
Patch6: shadow-4.12.3-selinux-perms.patch
# https://github.com/shadow-maint/shadow/commit/e0524e813a3bae2891b33a66f35876841c11cee7
Patch7: useradd-check-if-subid-range-exists-for-user.patch
# https://github.com/shadow-maint/shadow/commit/e0524e813a3bae2891b33a66f35876841c11cee7
Patch8: Fix-HAVE_SHADOWGRP-configure-check.patch
# https://github.com/shadow-maint/shadow/commit/8cce4557e00e9e556e49b10e604eea2e88d3846d
Patch9: Don-t-else-after-a-noreturn-call.patch
# https://github.com/shadow-maint/shadow/commit/155c9421b935202a550193b15dfd4eba014d21df
Patch10: libmisc-agetpass-erase_pass-Add-functions-for-gettin.patch
# https://github.com/shadow-maint/shadow/commit/554f86bafa2581d81ffb39e92240e39d630c43c4
Patch11: Replace-the-deprecated-getpass-3-by-our-agetpass.patch
# https://github.com/shadow-maint/shadow/commit/a5b8810bb1791ef044b9421969131bd385b8a70
Patch12: agetpass-Hook-into-build-system.patch
# https://github.com/shadow-maint/shadow/commit/bc18c184e56ff9aa6b092b01116b46cea4f34492
Patch13: chfn-new_fields-fix-wrong-fields-printed.patch
# https://github.com/shadow-maint/shadow/commit/65470e5c7ddbc2885ad35fbd0218b91a75626c2f
Patch14: changing-lock-mechanism.patch
# https://github.com/shadow-maint/shadow/commit/1d936c968a213319eff46e78284c9dd45d6c79dd
Patch15: Warn-if-failed-to-read-existing-etc-nsswitch.conf.patch
# https://github.com/shadow-maint/shadow/commit/8e0ad48c21bd7d5506ff44eb4c04f796b80045ce
Patch16: Prevent-out-of-boundary-access.patch
# https://github.com/shadow-maint/shadow/commit/0ec157d579931cfec1cc8439288ed17adf19da5d
Patch17: Fix-memory-leaks-by-replacing-realloc-3-with-realloc.patch
# https://github.com/shadow-maint/shadow/commit/f301a4ca19533a5c93bba44db2d8e345fa909e35
Patch18: Handle-reallocf-3-errors.patch
# https://github.com/shadow-maint/shadow/commit/1482224c546cabc3a08ec069c775b116171f182a
Patch19: Use-freezero-3-where-suitable.patch
# https://github.com/shadow-maint/shadow/commit/5c5dc756419e1d3cad63c66c4bb803eadf1074fd
Patch20: libmisc-agetpass-Fix-bug-detecting-truncation.patch

BuildRequires:  audit-libs-devel
BuildRequires:  autoconf
BuildRequires:  automake
BuildRequires:  bison
BuildRequires:  docbook-dtds
BuildRequires:  docbook-style-xsl
BuildRequires:  flex
BuildRequires:  gcc
BuildRequires:  gettext-devel
BuildRequires:  itstool
BuildRequires:  libbsd-devel
BuildRequires:  libselinux-devel
BuildRequires:  libsemanage-devel
BuildRequires:  libtool
BuildRequires:  libxslt
BuildRequires:  make
BuildRequires:  pkgconf

Requires:       setup libselinux audit-libs

%description
The shadow-utils package includes the necessary programs for
converting UNIX password files to the shadow password format, plus
programs for managing user and group accounts. The pwconv command
converts passwords to the shadow password format. The pwunconv command
unconverts shadow passwords and generates a passwd file (a standard
UNIX password file). The pwck command checks the integrity of password
and shadow files. The lastlog command prints out the last login times
for all users. The useradd, userdel, and usermod commands are used for
managing user accounts. The groupadd, groupdel, and groupmod commands
are used for managing group accounts.

%package doc
Summary:        Documentation files for %{name}
Requires:       %{name} = %{version}-%{release}
BuildArch:      noarch

%description    doc
The %{name}-doc package contains documentation files for %{name}.

%package subid
Summary: A library to manage subordinate uid and gid ranges
License: BSD and GPLv2+

%description subid
Utility library that provides a way to manage subid ranges.

%package subid-devel
Summary: Development package for shadow-utils-subid
License: BSD and GPLv2+
Requires: shadow-utils-subid

%description subid-devel
Development files for shadow-utils-subid.

%prep
%autosetup -p1 -n shadow-%{version}

iconv -f ISO88591 -t utf-8  doc/HOWTO > doc/HOWTO.utf8
cp -f doc/HOWTO.utf8 doc/HOWTO

cp -a %{SOURCE3} %{SOURCE4} .

rm libmisc/getdate.c

%build
export CFLAGS="$RPM_OPT_FLAGS -fpie"
export LDFLAGS="-pie -Wl,-z,relro -Wl,-z,now"

autoreconf
%configure \
        --enable-shadowgrp \
        --enable-man \
        --with-audit \
        --with-sha-crypt \
        --with-bcrypt \
        --with-yescrypt \
        --with-selinux \
        --without-libcrack \
        --without-libpam \
        --enable-shared \
        --with-group-name-max-length=32
%make_build

%install
%make_install gnulocaledir=$RPM_BUILD_ROOT%{_datadir}/locale MKINSTALLDIRS=`pwd`/mkinstalldirs
install -d -m 755 $RPM_BUILD_ROOT%{_sysconfdir}/default
install -p -c -m 0644 %{SOURCE2} $RPM_BUILD_ROOT%{_sysconfdir}/login.defs
install -p -c -m 0600 %{SOURCE1} $RPM_BUILD_ROOT%{_sysconfdir}/default/useradd


ln -s useradd $RPM_BUILD_ROOT%{_sbindir}/adduser
ln -s useradd.8 $RPM_BUILD_ROOT%{_mandir}/man8/adduser.8
for subdir in $RPM_BUILD_ROOT%{_mandir}/{??,??_??,??_??.*}/man* ; do
        test -d $subdir && test -e $subdir/useradd.8 && echo ".so man8/useradd.8" > $subdir/adduser.8
done

rm $RPM_BUILD_ROOT%{_bindir}/{chfn,chsh,expiry,groups,login,passwd,su,faillog}
rm $RPM_BUILD_ROOT%{_sbindir}/{logoutd,nologin}
rm $RPM_BUILD_ROOT%{_sysconfdir}/{login.access,limits}
find $RPM_BUILD_ROOT%{_mandir} -type f -regex '.*man.*\/\(chfn\|chsh\|login\.access\|expiry\|groups\|login\|passwd\|su\|limits\|porttime\|suauth\|logoutd\|nologin\|getspnam\|faillog\)\.[1-9].*' -print -delete
find $RPM_BUILD_ROOT%{_mandir} -depth -type d -empty -delete

%find_lang shadow
for dir in $(ls -1d $RPM_BUILD_ROOT%{_mandir}/{??,??_??}) ; do
    dir=$(echo $dir | sed -e "s|^$RPM_BUILD_ROOT||")
    lang=$(basename $dir)
    echo "%%lang($lang) $dir/man*/*" >> shadow.filelist
done

# Move header files to its own folder
mkdir -p $RPM_BUILD_ROOT/%{includesubiddir}
install -m 644 libsubid/subid.h $RPM_BUILD_ROOT/%{includesubiddir}/

rm -f $RPM_BUILD_ROOT/%{_libdir}/libsubid.{la,a}
rm -f $RPM_BUILD_ROOT/%{_datadir}/locale/*/LC_MESSAGES/shadow.mo

%generate_compatibility_deps

%files -f shadow.filelist
%dir %{abidir}
%license gpl-2.0.txt shadow-bsd.txt
%attr(0644,root,root)   %config(noreplace) %{_sysconfdir}/login.defs
%attr(0644,root,root)   %config(noreplace) %{_sysconfdir}/default/useradd
%{_bindir}/sg
%attr(4755,root,root) %{_bindir}/chage
%attr(4755,root,root) %{_bindir}/gpasswd
%{_bindir}/lastlog
%attr(4755,root,root) %{_bindir}/newgrp
%attr(0755,root,root) %caps(cap_setgid=ep) %{_bindir}/newgidmap
%attr(0755,root,root) %caps(cap_setuid=ep) %{_bindir}/newuidmap
%{_sbindir}/adduser
%attr(0755,root,root)   %{_sbindir}/user*
%attr(0755,root,root)   %{_sbindir}/group*
%{_sbindir}/grpck
%{_sbindir}/pwck
%{_sbindir}/*conv
%{_sbindir}/chpasswd
%{_sbindir}/chgpasswd
%{_sbindir}/newusers
%{_sbindir}/vipw
%{_sbindir}/vigr
%{abidir}/*-option.list
%{_mandir}/man1/chage.1*
%{_mandir}/man1/gpasswd.1*
%{_mandir}/man1/sg.1*
%{_mandir}/man1/newgrp.1*
%{_mandir}/man1/newgidmap.1*
%{_mandir}/man1/newuidmap.1*
%{_mandir}/man3/shadow.3*
%{_mandir}/man5/shadow.5*
%{_mandir}/man5/login.defs.5*
%{_mandir}/man5/gshadow.5*
%{_mandir}/man5/subuid.5*
%{_mandir}/man5/subgid.5*
%{_mandir}/man8/adduser.8*
%{_mandir}/man8/group*.8*
%{_mandir}/man8/user*.8*
%{_mandir}/man8/pwck.8*
%{_mandir}/man8/grpck.8*
%{_mandir}/man8/chpasswd.8*
%{_mandir}/man8/chgpasswd.8*
%{_mandir}/man8/newusers.8*
%{_mandir}/man8/*conv.8*
%{_mandir}/man8/lastlog.8*
%{_mandir}/man8/vipw.8*
%{_mandir}/man8/vigr.8*

%files doc
%doc NEWS doc/HOWTO README

%files subid
%{_libdir}/libsubid.so.*
%{abidir}/libsubid*.dump
%{_bindir}/getsubids
%{_mandir}/man1/getsubids.1*

%files subid-devel
%{_includedir}/shadow/subid.h
%{_libdir}/libsubid.so


%changelog
* Wed Feb 22 2023 Shawn Wang <shawnwang@linux.alibaba.com> - 4.13-2
- fix some bugs for 4.13

* Sun Jan 08 2023 Shawn Wang <shawnwang@linux.alibaba.com> - 4.13-1
- update to 4.13
- remove some unused build requires
- add some patches from upstream

* Fri Oct 28 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 4.12.3-4
- chage: Fix regression in print_date

* Thu Oct 13 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 4.12.3-3
- optimise spec file

* Fri Sep 30 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 4.12.3-2
- add nologin patch

* Fri Sep 23 2022 mgb01105731 <mgb01105731@alibaba-inc.com> - 4.12.3-1
- update to 4.12.3

* Thu Mar 24 2022 Chunmei Xu <xuchunmei@linux.alibaba.com> - 4.11.1-3
- fix /etc/login.defs wrong content

* Fri Mar 18 2022 forrest_ly <flin@linux.alibaba.com> - 4.11.1-2
- Add upstream patches

* Wed Mar  2 2022 forrest_ly <flin@linux.alibaba.com> - 4.11.1-1
- Init for Anolis OS 23
